Type: Package
Package: ShinyItemAnalysis
Title: Test and Item Analysis via Shiny
Version: 1.6.0
Date: 2026-06-08
Authors@R: c(
    person("Patricia", "Martinkova", , "martinkova@cs.cas.cz", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-4754-8543")),
    person("Adela", "Hladka", role = "aut",
           comment = c(ORCID = "0000-0002-9112-1208")),
    person("Jan", "Netik", role = "aut",
           comment = c(ORCID = "0000-0002-3888-3203")),
    person("Ondrej", "Leder", role = "ctb"),
    person("Jakub", "Houdek", role = "ctb"),
    person("Lubomir", "Stepanek", role = "ctb"),
    person("Tomas", "Jurica", role = "ctb"),
    person("Jana", "Vorlickova", role = "ctb")
  )
Description: Package including functions and interactive shiny application
    for the psychometric analysis of educational tests, psychological
    assessments, health-related and other types of multi-item
    measurements, or ratings from multiple raters.
License: GPL-3
URL: https://shinyitemanalysis.org/
BugReports: https://github.com/patriciamar/ShinyItemAnalysis/issues
Depends: R (>= 4.1.0)
Imports: difR (>= 5.0), dplyr, ggplot2 (>= 3.5.0), lme4, methods, mirt
        (>= 1.43), nnet, psych (>= 2.1.9), purrr, rlang, rstudioapi,
        tibble, tidyr (>= 1.3.0)
Suggests: cachem, data.table, deltaPlotR, difNLR (>= 1.5.0), DT,
        ggdendro, gridExtra, knitr, latticeExtra, ltm, plotly,
        rmarkdown, shiny (>= 1.6.0), shinyBS, shinyjs, shinytest2,
        stringr, testthat (>= 3.0.0), VGAM, xtable, yaml
Config/testthat/edition: 3
Encoding: UTF-8
LazyData: true
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-07 22:02:28 UTC; martinkova
Author: Patricia Martinkova [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-4754-8543>),
  Adela Hladka [aut] (ORCID: <https://orcid.org/0000-0002-9112-1208>),
  Jan Netik [aut] (ORCID: <https://orcid.org/0000-0002-3888-3203>),
  Ondrej Leder [ctb],
  Jakub Houdek [ctb],
  Lubomir Stepanek [ctb],
  Tomas Jurica [ctb],
  Jana Vorlickova [ctb]
Maintainer: Patricia Martinkova <martinkova@cs.cas.cz>
Repository: CRAN
Date/Publication: 2026-06-08 07:30:02 UTC
Built: R 4.5.2; ; 2026-06-08 08:20:38 UTC; unix
