# Generated by roxygen2: do not edit by hand

S3method(plot,sia_parallel)
S3method(print,blis_coefs)
export(DDplot)
export(DistractorAnalysis)
export(ICCrestricted)
export(ItemAnalysis)
export(blis)
export(fa_parallel)
export(fit_blis)
export(gDiscrim)
export(get_orig_levels)
export(ggWrightMap)
export(nominal_to_int)
export(obtain_nrm_def)
export(plotAdjacent)
export(plotCumulative)
export(plotDIFLogistic)
export(plotDIFirt)
export(plotDistractorAnalysis)
export(plotMultinomial)
export(plot_corr)
export(recode_nr)
export(run_app)
export(startShinyItemAnalysis)
export(theme_app)
exportClasses(BlisClass)
exportMethods(coef)
importFrom(difR,LordChi2)
importFrom(difR,RajuZ)
importFrom(difR,itemParEst)
importFrom(difR,itemRescale)
importFrom(difR,mantelHaenszel)
importFrom(difR,sibTest)
importFrom(dplyr,arrange)
importFrom(dplyr,group_by)
importFrom(dplyr,mutate)
importFrom(dplyr,relocate)
importFrom(dplyr,row_number)
importFrom(dplyr,ungroup)
importFrom(ggplot2,aes)
importFrom(ggplot2,annotate)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,coord_fixed)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,expansion)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_ribbon)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggplotGrob)
importFrom(ggplot2,ggplot_build)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,margin)
importFrom(ggplot2,scale_alpha_manual)
importFrom(ggplot2,scale_color_gradient2)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_colour_manual)
importFrom(ggplot2,scale_fill_gradient2)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_linetype_manual)
importFrom(ggplot2,scale_shape_manual)
importFrom(ggplot2,scale_size_area)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,scale_y_discrete)
importFrom(ggplot2,scale_y_reverse)
importFrom(ggplot2,stat_function)
importFrom(ggplot2,stat_summary)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,unit)
importFrom(ggplot2,update_labels)
importFrom(ggplot2,xlab)
importFrom(ggplot2,xlim)
importFrom(ggplot2,ylab)
importFrom(ggplot2,ylim)
importFrom(grDevices,hcl)
importFrom(grDevices,rainbow)
importFrom(grid,gTree)
importFrom(grid,grid.draw)
importFrom(grid,grid.newpage)
importFrom(lme4,VarCorr)
importFrom(lme4,bootMer)
importFrom(lme4,lmer)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,setClass)
importFrom(methods,setMethod)
importFrom(mirt,fscores)
importFrom(mirt,key2binary)
importFrom(mirt,mirt)
importFrom(nnet,multinom)
importFrom(parallel,mclapply)
importFrom(psych,alpha)
importFrom(psych,fa)
importFrom(psych,polychoric)
importFrom(psych,tetrachoric)
importFrom(purrr,keep)
importFrom(purrr,map)
importFrom(purrr,map2)
importFrom(purrr,map2_dbl)
importFrom(purrr,map_dfr)
importFrom(purrr,modify)
importFrom(purrr,modify2)
importFrom(purrr,modify_if)
importFrom(purrr,walk)
importFrom(rlang,.data)
importFrom(rlang,abort)
importFrom(rlang,arg_match)
importFrom(rlang,check_installed)
importFrom(rlang,cnd_muffle)
importFrom(rlang,inform)
importFrom(rlang,is_interactive)
importFrom(rlang,try_fetch)
importFrom(rstudioapi,executeCommand)
importFrom(rstudioapi,isAvailable)
importFrom(rstudioapi,jobRunScript)
importFrom(rstudioapi,jobSetStatus)
importFrom(stats,aggregate)
importFrom(stats,as.dist)
importFrom(stats,coef)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,cutree)
importFrom(stats,deriv)
importFrom(stats,deviance)
importFrom(stats,fitted)
importFrom(stats,glm)
importFrom(stats,hclust)
importFrom(stats,median)
importFrom(stats,na.exclude)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,qchisq)
importFrom(stats,qf)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,relevel)
importFrom(stats,rnorm)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,sigma)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(stats,xtabs)
importFrom(tibble,add_row)
importFrom(tibble,as_tibble)
importFrom(tibble,rowid_to_column)
importFrom(tibble,tibble)
importFrom(tidyr,drop_na)
importFrom(tidyr,nest)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,unnest)
importFrom(utils,available.packages)
importFrom(utils,capture.output)
importFrom(utils,data)
importFrom(utils,head)
importFrom(utils,install.packages)
importFrom(utils,installed.packages)
importFrom(utils,menu)
importFrom(utils,packageVersion)
importFrom(utils,read.csv)
